@@ -4,11 +4,11 @@ function(x, type=0)
     x <- as.matrix(x)
     x <- ifelse(x > 0, 1, 0)
     if (NCOL(x) < 2)
-        stop("provide at least 2 columns for 'x'")
+        stop("provide at least two columns for 'x'")
     if (!(type %in% 0:2))
-        stop("'type' must be in c(0, 1, 2)")
+        stop("'type' must be 0, 1 or 2")
     n <- nrow(x)
-    j <- t(x) %*% x
+    j <- crossprod(x) ## faster t(x) %*% x
     ip1 <- sweep(j, 1, diag(j), "/")
     ip2 <- 1 - sweep(-sweep(j, 2, diag(j), "-"), 1, n - diag(j), "/")
     out <- switch(as.character(type),
