@@ -3,14 +3,14 @@
              model = c("global", "local", "linear", "hybrid"),
              threshold = 0.8, maxit = 200, weakties = TRUE, stress = 1,
              scaling = TRUE, pc = TRUE, smin = 1e-4, sfgrmin = 1e-7,
-             sratmax=0.99999, ...) 
+             sratmax=0.99999, ...)
 {
     ## Check that 'dist' are distances or a symmetric square matrix
-    if (!(inherits(dist, "dist") || (is.matrix(dist) || is.data.frame(dist))
-          && ncol(dist) == nrow(dist)
-          && isTRUE(all.equal(dist[lower.tri(dist)], t(dist)[lower.tri(dist)]))))
+    if (!(inherits(dist, "dist") ||
+              ((is.matrix(dist) || is.data.frame(dist)) &&
+                   isSymmetric(unname(as.matrix(dist))))))
         stop("'dist' must be a distance object (class \"dist\") or a symmetric square matrix")
-    if (any(dist < -sqrt(.Machine$double.eps)))
+    if (any(dist < -sqrt(.Machine$double.eps), na.rm = TRUE))
         warning("some dissimilarities are negative -- is this intentional?")
     ## match.arg
     model <- match.arg(model)
@@ -104,7 +104,8 @@
         ities <- 1
     else
         ities <- 2
-    ## Fortran call
+    ## Fortran call -- must quoted because monoMDS has mixed case (or
+    ## can be FIXED)
     sol <- .Fortran("monoMDS", nobj = as.integer(nobj), nfix=as.integer(0),
                  ndim = as.integer(k), ndis = as.integer(ndis),
                  ngrp = as.integer(ngrp), diss = as.double(dist),
@@ -117,8 +118,7 @@
                  sfgrmn = as.double(sfgrmin), dist = double(ndis),
                  dhat = double(ndis), points = double(k*nobj),
                  stress = double(1), grstress = double(ngrp),
-                 iters = integer(1), icause = integer(1),
-                 PACKAGE = "vegan")
+                 iters = integer(1), icause = integer(1))
     sol$call <- match.call()
     sol$model <- model
     sol$points <- matrix(sol$points, nobj, k)
