@@ -1,10 +1,20 @@
 panel.ordi <-
-function(x, y, biplot, type = type,  ...)
+function(x, y, biplot, polygon, type = type, ...)
 {
-    panel.xyplot(x, y, type = type,  ...)
     tp <- trellis.par.get()
     sp <- tp$superpose.symbol
     ps <- tp$plot.symbol
+    ## ordixyplot passes polygon of all points, but ordisplom does not
+    if ("polygon" %in% type && !missing(polygon)) {
+        ppar <- tp$plot.polygon
+        lpolygon(polygon, col = ppar$col, border = ppar$border,
+                 alpha = ppar$alpha, lty = ppar$lty, lwd = ppar$lwd, ...)
+        inpol <- chull(x, y)
+        par <- tp$superpose.polygon
+        lpolygon(x[inpol], y[inpol], col = par$col, border = par$border,
+                 alpha = par$alpha, lty = par$lty, lwd = par$lwd, ...)
+    }
+    panel.xyplot(x, y, type = type,  ...)
     if ("biplot" %in% type && !is.null(biplot$arrows)) {
         panel.arrows(0, 0, biplot$arrows[,2], biplot$arrows[,1],
                      col=sp$col, ...)
@@ -16,7 +26,7 @@ function(x, y, biplot, type = type,  ...)
                      ...)
     }
     if ("arrows" %in% type) {
-        panel.superpose(x, y, panel.groups= "panel.ordiarrows",  ...)
+        panel.superpose(x, y, panel.groups= "panel.ordiarrows", ...)
     }
     panel.abline(h=0, lty = 3)
     panel.abline(v=0, lty = 3)
@@ -29,9 +39,15 @@ function(x, y, subscripts,
          ...)
 {
     n <- length(x)
+    col <- trellis.par.get("superpose.line")$col
+    group <- list(...)$group.number
+    if (is.null(group))
+        col <- col[1]
+    else
+        col = rep(col, len=group)[group]
     panel.arrows(x[-n], y[-n], x[-1], y[-1], ends = ends, type = "open",
                  length = length, angle = angle, code = code,
-                 col = trellis.par.get("superpose.line")$col,
+                 col = col,
                  lwd = trellis.par.get("superpose.line")$lwd,
                  )
 }
