@@ -70,3 +70,17 @@
     }
     invisible(x)
 }
+
+### package klaR has another function called rda(), and to avoid using
+### klar:::print.rda instead of delegating to vegan:::print.cca we
+### define here a (redundant) print.rda
+
+`print.rda` <-
+    function(x, ...)
+{
+    ## not vegan rda?
+    if (!("CA" %in% names(x)))
+        stop(gettextf("%s is not a vegan rda object",
+                      sQuote(deparse(substitute(x)))))
+    NextMethod("print", x, ...)
+}
