@@ -1,17 +1,17 @@
 `simulate.rda` <-
     function(object, nsim = 1, seed = NULL, indx = NULL, rank = "full",
-             correlated = FALSE, ...) 
+             correlated = FALSE, ...)
 {
     ## Fail if there is no constrained component (it could be possible
     ## to change the function to handle unconstrained ordination, too,
     ## when rank < "full", but that would require redesign)
     if (is.null(object$CCA))
         stop("function can be used only with constrained ordination")
-    
+
     ## Handle RNG: code directly from stats::simulate.lm
-    if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) 
+    if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE))
         runif(1)
-    if (is.null(seed)) 
+    if (is.null(seed))
         RNGstate <- get(".Random.seed", envir = .GlobalEnv)
     else {
         R.seed <- get(".Random.seed", envir = .GlobalEnv)
@@ -33,39 +33,62 @@
         if(nrow(indx) != nsim)
             stop(gettextf("'nsim' (%d) and no. of 'indx' rows (%d) do not match",
                           nsim, nrow(indx)))
+    ## collect data to back-transform data to the scale of observations
+    sqnr1 <- sqrt(nobs(object) - 1)
+    ## the ifs are only needed to cope with pre-2.5-0 vegan: now
+    ## we always have Ybar, but earlier we needed to check whether
+    ## we had CA or CCA Xbar
+    if (!is.null(object$Ybar)) {
+        cnt <- attr(object$Ybar, "scaled:center")
+        scl <- attr(object$Ybar, "scaled:scale")
+    } else { # needed for vegan-2.4 compatibility
+        if (is.null(object$CCA))
+            tmp <- object$CA$Xbar
+        else tmp <- object$CCA$Xbar
+        cnt <- attr(tmp, "scaled:center")
+        scl <- attr(tmp, "scaled:scale")
+    }
+
     ## Proper simulation: very similar for simulate.lm, but produces
     ## an array of response matrices
 
-    ftd <- predict(object, type = "response", rank = rank)
+    ftd <- predict(object, type = "working", rank = rank)
     ## pRDA: add partial Fit to the constrained
     if (!is.null(object$pCCA))
-        ftd <- ftd + object$pCCA$Fit
+        ftd <- ftd + ordiYbar(object, "pCCA")
     ## if(is.null(indx)), we have parametric Gaussian simulation and
     ## need to generate sd matrices. The residuals sd is always taken
-    ## from the unconstrained (residual) component $CA$Xbar. If
+
+    ## from the unconstrained (residual) component. If
     ## species are uncorrelated, we need only species sd's, but if
     ## correlated, we also need species covariances.
+    CAYbar <- ordiYbar(object, "CA")
     if (!correlated)
-        dev <- outer(rep(1, nrow(ftd)), apply(object$CA$Xbar, 2, sd))
+        dev <- outer(rep(1, nrow(ftd)), apply(CAYbar, 2, sd))
     else
-        dev <- cov(object$CA$Xbar)
+        dev <- cov(CAYbar)
     ## Generate an array
     ans <- array(0, c(dim(ftd), nsim))
     for (i in seq_len(nsim)) {
         if (!is.null(indx))
-            ans[,,i] <- as.matrix(ftd + object$CA$Xbar[indx[i,],])
+            ans[,,i] <- as.matrix(ftd + CAYbar[indx[i,],])
         else if (!correlated)
             ans[,,i] <- as.matrix(ftd + matrix(rnorm(length(ftd), sd = dev),
                                                nrow = nrow(ftd)))
         else {
             ans[,,i] <- t(apply(ftd, 1,
-                                function(x) mvrnorm(1, mu = x, Sigma = dev))) 
+                                function(x) mvrnorm(1, mu = x, Sigma = dev)))
         }
+        ## ans to the scale of observations
+        ans[,,i] <- ans[,,i] * sqnr1
+        if (!is.null(scl))
+            ans[,,i] <- sweep(ans[,,i], 2, scl, "*")
+        ans[,,i] <- sweep(ans[,,i], 2, cnt, "+")
     }
     ## set RNG attributes
     if (is.null(indx))
         attr(ans, "seed") <- RNGstate
-    else 
+    else
         attr(ans, "seed") <- "index"
     ## set commsim attributes if nsim > 1, else return a 2-dim matrix
     if (nsim == 1) {
@@ -74,7 +97,7 @@
     } else {
         dimnames(ans) <- list(rownames(ftd), colnames(ftd),
                               paste("sim", seq_len(nsim), sep = "_"))
-        attr(ans, "data") <- round(ftd + object$CA$Xbar, 12)
+        attr(ans, "data") <- round(ftd + CAYbar, 12)
         attr(ans, "method") <- paste("simulate", ifelse(is.null(indx),
                                                         "parametric", "index"))
         attr(ans, "binary") <- FALSE
@@ -101,9 +124,9 @@
     if (is.null(object$CCA))
         stop("function can be used only with constrained ordination")
     ## Handle RNG: code directly from stats::simulate.lm
-    if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) 
+    if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE))
         runif(1)
-    if (is.null(seed)) 
+    if (is.null(seed))
         RNGstate <- get(".Random.seed", envir = .GlobalEnv)
     else {
         R.seed <- get(".Random.seed", envir = .GlobalEnv)
@@ -120,16 +143,16 @@
     if (!is.null(indx))
         if(nrow(indx) != nsim)
             stop(gettextf("'nsim' (%d) and no. of 'indx' rows (%d) do not match",
-                          nsim, nrow(indx)))   
+                          nsim, nrow(indx)))
     ## Need sqrt of rowsums for weighting
     sq.r <- sqrt(object$rowsum)
     ## Fitted value
     ftd <- predict(object, type = "working", rank = rank)
     ## pCCA: add partial Fit to the constrained
     if (!is.null(object$pCCA))
-        ftd <- ftd + object$pCCA$Fit
+        ftd <- ftd + ordiYbar(object, "pCCA")
     ## Residual Xbar need weighting and back-weighting
-    Xbar <- sweep(object$CA$Xbar, 1, sq.r, "*")
+    Xbar <- sweep(ordiYbar(object, "CA"), 1, sq.r, "*")
     ## Simulation
     if (correlated)
         dev <- cov(Xbar)
@@ -141,11 +164,11 @@
             if (correlated)
                 tmp <- mvrnorm(nrow(ftd), numeric(ncol(ftd)), Sigma = dev)
             else
-                tmp <- matrix(rnorm(length(ftd), sd = dev), 
+                tmp <- matrix(rnorm(length(ftd), sd = dev),
                           nrow = nrow(ftd))
             ans[,,i] <- as.matrix(ftd + sweep(tmp, 1, sq.r, "/"))
         }
-        else 
+        else
             ans[,,i] <- as.matrix(ftd + sweep(Xbar[indx[i,],], 1, sq.r, "/"))
     }
     ## From internal form to the original form with fixed marginal totals
@@ -164,7 +187,7 @@
     } else {
         dimnames(ans) <- list(rownames(ftd), colnames(ftd),
                               paste("sim", seq_len(nsim), sep = "_"))
-        obsdata <- ftd + object$CA$Xbar
+        obsdata <- ordiYbar(object, "initial")
         obsdata <- (obsdata * sqrt(rc) + rc) * object$grand.total
         attr(ans, "data") <- round(obsdata, 12)
         attr(ans, "method") <- paste("simulate", ifelse(is.null(indx),
@@ -176,7 +199,7 @@
         attr(ans, "end") <- as.integer(nsim)
         attr(ans, "thin") <- 1L
         class(ans) <- c("simulate.cca", "simmat", "array")
-    }    
+    }
     ans
 }
 
@@ -190,7 +213,7 @@
 
 `simulate.capscale` <-
     function(object, nsim = 1, seed = NULL, indx = NULL, rank = "full",
-             correlated = FALSE, ...) 
+             correlated = FALSE, ...)
 {
     ## Fail if no CCA component
     if (is.null(object$CCA))
@@ -198,9 +221,9 @@
     if (is.null(indx) && correlated)
         warning("argument 'correlated' does not work and will be ignored")
     ## Handle RNG: code directly from stats::simulate.lm
-    if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) 
+    if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE))
         runif(1)
-    if (is.null(seed)) 
+    if (is.null(seed))
         RNGstate <- get(".Random.seed", envir = .GlobalEnv)
     else {
         R.seed <- get(".Random.seed", envir = .GlobalEnv)
@@ -215,30 +238,29 @@
     ## constrained solution, and we need rank of the data (not of
     ## constraints).
     if (rank > 0) {
-        ftd <- qr.fitted(object$CCA$QR, object$CCA$Xbar)
+        ftd <- ordiYbar(object, "CCA")
         ## redo analysis when rank < full
         if (rank < object$CCA$rank) {
             x <- svd(ftd, nu = rank, nv = rank)
             ftd <- x$u %*% diag(x$d[1:rank], nrow=rank) %*% t(x$v)
         }
     } else {
-        ftd <- matrix(0, nrow=nrow(object$CA$Xbar),
-                      ncol = ncol(object$CA$Xbar))
+        ftd <- 0
     }
     ## add partial Fit to the constrained
     if (!is.null(object$pCCA))
-        ftd <- ftd + object$pCCA$Fit
+        ftd <- ftd + ordiYbar(object, "pCCA")
     if (is.null(indx))
-        ans <- as.data.frame(ftd + matrix(rnorm(length(ftd), 
-               sd = outer(rep(1,nrow(ftd)), apply(object$CA$Xbar, 2, sd))), 
+        ans <- as.data.frame(ftd + matrix(rnorm(length(ftd),
+               sd = outer(rep(1,nrow(ftd)), apply(ordiYbar(object, "CA"), 2, sd))),
                nrow = nrow(ftd)))
     else
-        ans <- ftd + object$CA$Xbar[indx,]
+        ans <- ftd + ordiYbar(object, "CA")[indx,]
     ## return Euclidean distances
+    ans <- ans * object$adjust
     ans <- dist(ans)
     ## remove adjustment done in capscale and put dissimilarities to
     ## (approximately) original scale
-    ans <- ans/object$adjust
     if (is.null(indx))
         attr(ans, "seed") <- RNGstate
     else
